@@ -24,19 +24,14 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#if !defined (GTK_DISABLE_DEPRECATED) || defined (__GTK_LIST_C__)
+
 #ifndef __GTK_LIST_H__
 #define __GTK_LIST_H__
 
+#include <gtk/gtk.h>
 
-#include <gdk/gdk.h>
-#include <gtk/gtkenums.h>
-#include <gtk/gtkcontainer.h>
-#include <gtk/gtklistitem.h>
-
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
-
+G_BEGIN_DECLS
 
 #define GTK_TYPE_LIST                  (gtk_list_get_type ())
 #define GTK_LIST(obj)                  (GTK_CHECK_CAST ((obj), GTK_TYPE_LIST, GtkList))
@@ -136,9 +131,8 @@ void       gtk_list_toggle_row            (GtkList          *list,
 void       gtk_list_undo_selection        (GtkList          *list);
 void       gtk_list_end_drag_selection    (GtkList          *list);
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
-
+G_END_DECLS
 
 #endif /* __GTK_LIST_H__ */
+
+#endif /* GTK_DISABLE_DEPRECATED */
