@@ -902,8 +902,8 @@ setup_msw_rc_style (void)
 	      "ythickness = %d\n"
 	      "}\n"
 	      "class \"GtkComboBox\" style \"msw-combobox\"\n",
-        xp_theme_is_active ()? 1 : GetSystemMetrics (SM_CXEDGE),
-        xp_theme_is_active ()? 1 : GetSystemMetrics (SM_CYEDGE));
+        xp_theme_is_active()? 1 : GetSystemMetrics (SM_CXEDGE),
+        xp_theme_is_active()? 1 : GetSystemMetrics (SM_CYEDGE));
   gtk_rc_parse_string (buf);
 
   /* size of tree view header */
@@ -912,8 +912,8 @@ setup_msw_rc_style (void)
 	      "{\n"
 	      "xthickness = 0\n"
 	      "ythickness = 0\n"
-	      "GtkWidget::draw-border = { 0, 0, 0, 0 }\n"
-	      "GtkButton::default-border = { 0, 0, 0, 0 }\n"
+	      "GtkWidget::draw-border = {0, 0, 0, 0}\n"
+        "GtkButton::default-border = { 0, 0, 0, 0 }\n"
 	      "GtkButton::default-outside-border = { 0, 0, 0, 0 }\n"
 	      "GtkButton::child-displacement-x = 0\n"
 	      "GtkButton::child-displacement-y = 0\n"
@@ -1097,7 +1097,8 @@ combo_box_draw_arrow (GtkStyle *style,
 
       dc = get_window_dc (style, window, state, area->x, area->y, area->width,
 			  area->height, &rect);
-      border = (GTK_TOGGLE_BUTTON (widget->parent)->active ? DFCS_PUSHED | DFCS_FLAT : 0);
+      border = (GTK_TOGGLE_BUTTON (widget->parent)->
+		active ? DFCS_PUSHED | DFCS_FLAT : 0);
 
       InflateRect (&rect, 1, 1);
       DrawFrameControl (dc, &rect, DFC_SCROLL, DFCS_SCROLLDOWN | border);
@@ -1596,7 +1597,8 @@ draw_arrow (GtkStyle *style,
       /* draw the toolbar chevrons - waiting for GTK 2.4 */
       if (name && !strcmp (name, "gtk-toolbar-arrow"))
 	{
-	  if (xp_theme_draw (window, XP_THEME_ELEMENT_REBAR_CHEVRON, style, x, y,
+	  if (xp_theme_draw
+	      (window, XP_THEME_ELEMENT_REBAR_CHEVRON, style, x, y,
 	       width, height, state, area))
 	    {
 	      return;
@@ -1606,7 +1608,8 @@ draw_arrow (GtkStyle *style,
       else if (0		/* widget->parent && GTK_IS_BUTTON
 				   (widget->parent) */ )
 	{
-	  if (xp_theme_draw (window, XP_THEME_ELEMENT_COMBOBUTTON, style, x - 3,
+	  if (xp_theme_draw
+	      (window, XP_THEME_ELEMENT_COMBOBUTTON, style, x - 3,
 	       widget->allocation.y + 1, width + 5,
 	       widget->allocation.height - 4, state, area))
 	    {
@@ -1978,26 +1981,28 @@ draw_box (GtkStyle *style,
       release_window_dc (style, window, state_type);
 
       if (xp_theme_is_active ()
-          && xp_theme_draw (window, XP_THEME_ELEMENT_COMBOBUTTON, style, x, y,
-          width, height, state_type, area))
-        {
-          cx = GetSystemMetrics(SM_CXVSCROLL);
-          x += width - cx;
-          width = cx;
+	  && xp_theme_draw (window, XP_THEME_ELEMENT_COMBOBUTTON, style, x, y,
+			    width, height, state_type, area))
+	{
+      cx = GetSystemMetrics(SM_CXVSCROLL);
+      x += width - cx;
+      width = cx;
 
-          dc = get_window_dc (style, window, state_type, x, y, width - cx, height, &rect);
-          FillRect (dc, &rect, GetSysColorBrush (COLOR_WINDOW));
-          release_window_dc (style, window, state_type);
-          return;
-	      }
+
+      dc = get_window_dc (style, window, state_type, x, y, width - cx, height, &rect);
+      FillRect (dc, &rect, GetSysColorBrush (COLOR_WINDOW));
+      release_window_dc (style, window, state_type);
+      return;
+	}
     }
 
   if (detail &&
       (!strcmp (detail, "button") || !strcmp (detail, "buttondefault")))
     {
       if (GTK_IS_TREE_VIEW (widget->parent) || GTK_IS_CLIST (widget->parent))
-	{
-	  if (xp_theme_draw (window, XP_THEME_ELEMENT_LIST_HEADER, style, x, y,
+      {
+        if (xp_theme_draw
+	      (window, XP_THEME_ELEMENT_LIST_HEADER, style, x, y,
 	       width, height, state_type, area))
 	    {
 	      return;
@@ -2016,8 +2021,8 @@ draw_box (GtkStyle *style,
 	    }
 	}
       else if (is_toolbar_child (widget->parent)
-	       || (GTK_RELIEF_NONE ==
-		   gtk_button_get_relief (GTK_BUTTON (widget))))
+	       || (!GTK_IS_BUTTON (widget) ||
+		   (GTK_RELIEF_NONE == gtk_button_get_relief (GTK_BUTTON (widget)))))
 	{
 	  if (draw_tool_button (window, widget, style, x, y,
 				width, height, state_type, area))
@@ -2028,10 +2033,11 @@ draw_box (GtkStyle *style,
       else
 	{
 	  gboolean is_default = GTK_WIDGET_HAS_DEFAULT (widget);
-	  if (xp_theme_draw (window,
-			     is_default ? XP_THEME_ELEMENT_DEFAULT_BUTTON :
-			     XP_THEME_ELEMENT_BUTTON, style, x, y, width, height,
-			     state_type, area))
+	  if (xp_theme_draw
+	      (window,
+	       is_default ? XP_THEME_ELEMENT_DEFAULT_BUTTON :
+	       XP_THEME_ELEMENT_BUTTON, style, x, y, width, height,
+	       state_type, area))
 	    {
 	      return;
 	    }
@@ -2080,7 +2086,9 @@ draw_box (GtkStyle *style,
 	  gboolean is_v = GTK_IS_VSCROLLBAR (widget);
 
 	  if (xp_theme_draw (window,
-			     is_v ? XP_THEME_ELEMENT_SCROLLBAR_V : XP_THEME_ELEMENT_SCROLLBAR_H,
+			     is_v
+			     ? XP_THEME_ELEMENT_SCROLLBAR_V
+			     : XP_THEME_ELEMENT_SCROLLBAR_H,
 			     style, x, y, width, height, state_type, area))
 	    {
 	      XpThemeElement gripper =
@@ -2148,7 +2156,8 @@ draw_box (GtkStyle *style,
 	  GtkProgressBar *progress_bar = GTK_PROGRESS_BAR (widget);
 	  XpThemeElement xp_progress_bar =
 	    map_gtk_progress_bar_to_xp (progress_bar, TRUE);
-	  if (xp_theme_draw (window, xp_progress_bar, style, x, y, width, height,
+	  if (xp_theme_draw
+	      (window, xp_progress_bar, style, x, y, width, height,
 	       state_type, area))
 	    {
 	      return;
@@ -2163,7 +2172,9 @@ draw_box (GtkStyle *style,
 	  gboolean is_vertical = GTK_IS_VSCROLLBAR (widget);
 
 	  if (xp_theme_draw (window,
-			     is_vertical ? XP_THEME_ELEMENT_TROUGH_V : XP_THEME_ELEMENT_TROUGH_H,
+			     is_vertical
+			     ? XP_THEME_ELEMENT_TROUGH_V
+			     : XP_THEME_ELEMENT_TROUGH_H,
 			     style, x, y, width, height, state_type, area))
 	    {
 	      return;
@@ -2198,9 +2209,10 @@ draw_box (GtkStyle *style,
 
 	  if (is_vertical)
 	    {
-	      if (xp_theme_draw (window, XP_THEME_ELEMENT_SCALE_TROUGH_V,
-				 style, (2 * x + width) / 2, y, 2, height,
-				 state_type, area))
+	      if (xp_theme_draw
+		  (window, XP_THEME_ELEMENT_SCALE_TROUGH_V,
+		   style, (2 * x + width) / 2, y, 2, height,
+		   state_type, area))
 		{
 		  return;
 		}
@@ -2212,9 +2224,10 @@ draw_box (GtkStyle *style,
 	    }
 	  else
 	    {
-	      if (xp_theme_draw (window, XP_THEME_ELEMENT_SCALE_TROUGH_H,
-				 style, x, (2 * y + height) / 2, width, 2,
-				 state_type, area))
+	      if (xp_theme_draw
+		  (window, XP_THEME_ELEMENT_SCALE_TROUGH_H,
+		   style, x, (2 * y + height) / 2, width, 2,
+		   state_type, area))
 		{
 		  return;
 		}
@@ -2278,8 +2291,9 @@ draw_box (GtkStyle *style,
 
       if (name && !strcmp (name, "gtk-tooltips"))
 	{
-	  if (xp_theme_draw (window, XP_THEME_ELEMENT_TOOLTIP, style, x, y, width,
-			     height, state_type, area))
+	  if (xp_theme_draw
+	      (window, XP_THEME_ELEMENT_TOOLTIP, style, x, y, width,
+	       height, state_type, area))
 	    {
 	      return;
 	    }
@@ -2940,8 +2954,9 @@ draw_shadow (GtkStyle *style,
       HDC dc;
       RECT rect;
 
-      dc = get_window_dc (style, window, state_type, x, y, width, height, &rect);
 
+
+      dc = get_window_dc (style, window, state_type, x, y, width, height, &rect);
       if (is_combo_box_child (widget))
         {
           FillRect (dc, &rect, GetSysColorBrush (COLOR_WINDOW));
@@ -3146,7 +3161,8 @@ draw_hline (GtkStyle *style,
 {
   if (xp_theme_is_active () && detail && !strcmp (detail, "menuitem"))
     {
-      if (xp_theme_draw (window, XP_THEME_ELEMENT_MENU_SEPARATOR, style, x1, y, x2, 1,
+      if (xp_theme_draw
+	  (window, XP_THEME_ELEMENT_MENU_SEPARATOR, style, x1, y, x2, 1,
 	   state_type, area))
 	{
 	  return;
@@ -3263,8 +3279,9 @@ draw_resize_grip (GtkStyle *style,
 {
   if (detail && !strcmp (detail, "statusbar"))
     {
-      if (xp_theme_draw (window, XP_THEME_ELEMENT_STATUS_GRIPPER, style, x, y, width,
-			 height, state_type, area))
+      if (xp_theme_draw
+	  (window, XP_THEME_ELEMENT_STATUS_GRIPPER, style, x, y, width,
+	   height, state_type, area))
 	{
 	  return;
 	}
